from copy import deepcopy
import csv
import dateutil
from io import StringIO

from Acquisition import aq_inner
from Products.Five.browser import BrowserView
from Products.CMFCore.utils import getToolByName

from plone.app.textfield.interfaces import ITransformer
from plone.app.textfield.value import RichTextValue

from Products.Poi.adapters import IResponseContainer


class CSVExport(BrowserView):
    """ view to produce the CSV exports
    """

    def __call__(self):
        context = aq_inner(self.context)
        encoding = self.request.get('encoding')
        issuefolder = context.restrictedTraverse('@@issuefolder')

        issues = issuefolder.getFilteredIssues(self.request)
        buffer = StringIO()
        self.write_csv(issues, buffer)

        value = buffer.getvalue()

        if not encoding:
            encoding = 'UTF-8'
        self.request.response.setHeader('Content-type',
                                        'text/csv;charset='+encoding)
        self.request.response.setHeader('Content-Disposition',
                                        'attachment; filename=export.csv')

        return value

    def write_csv(self, issues, buffer):
        """
        Write the CSV for the given issues to the given file-like object.
        """
        pas_member = self.context.restrictedTraverse('@@pas_member')

        writer = csv.DictWriter(buffer, [
            'No.',
            'Title',
            'Status',
            'Owner',
            'IT Management Comment',
            'Details',
            'Recreation Steps',
            'Relevant Area',
            'Issue Type',
            'Severity',
            'Complexity',
            'Tags',
            'Last modified by',
            'Last modified by date/time',
            'Submitted by',
            'Submitted date/time',
            'Contact Email Address',
            'Software Engineer - Estimated',
            'Business Analyst Total Time',
            'Transend EDI Total Support Time',
            'System Engineer - Actual',
            'Operations ROI - Estimated',
            'Release Version',
            'Target Release',
        ])
        writer.writeheader()

        # to get the previous person who changed something
        # we need to get workflow and revision history
        mt = getToolByName(self.context, 'portal_membership')
        for issue in issues:
            obj = issue.getObject()

            # format rich text fields to play nicely with csv
            transformer = ITransformer(obj)
            if obj.details:
                details = deepcopy(obj.details)
                if type(obj.details) == str:
                    details = RichTextValue(details, 'text/html')
                transformedDetails = transformer(details, 'text/plain')
            else:
                transformedDetails = None

            if obj.steps:
                steps = deepcopy(obj.steps)
                if type(obj.steps) == str:
                    steps = RichTextValue(steps, 'text/html')
                transformedSteps = transformer(steps, 'text/plain')
            else:
                transformedSteps = None

            if hasattr(obj, 'last_actor'):
                last_actor = obj.last_actor
            else:
                responsefolder = IResponseContainer(obj)
                responses = []
                for id, response in enumerate(responsefolder):
                    if response is None:
                        # Has been removed.
                        continue
                    responses.append({'creator': response.creator})
                # the responses are in order so we just grab the last one
                if len(responses) > 0:
                    last_actor = responses[-1]['creator']
                else:
                    last_actor = issue.Creator
            actor_info = mt.getMemberInfo(last_actor)
            if actor_info and actor_info.get("fullname", None):
                last_actor = actor_info["fullname"]

            writer.writerow({
                'No.': issue.id,
                'Title': issue.Title,
                'Target Release': issue.target_release or "",
                'Relevant Area': obj.display_area(),
                'Issue Type': obj.display_issue_type(),
                'Severity': issue.severity,
                'Complexity': issue.complexity or '',
                'Owner': issue.assignee and pas_member.info(
                    issue.assignee)['fullname'],
                'IT Management Comment': obj.it_management_comment or "",
                'Tags': ", ".join(sorted(
                    (y for y in issue.Subject), key=lambda x: x.lower())),
                'Status': obj.getReviewState()['title'],
                'Details': transformedDetails or "",
                'Recreation Steps': transformedSteps or "",
                'Last modified by': last_actor,
                'Last modified by date/time': dateutil.parser.parse(
                    issue.ModificationDate).strftime('%Y-%m-%d %H:%M:%S'),
                'Release Version': issue.release or "",
                'Submitted by': pas_member.info(
                    issue.Creator)['name_or_id'],
                'Submitted date/time': dateutil.parser.parse(
                    issue.CreationDate).strftime('%Y-%m-%d %H:%M:%S'),
                'Contact Email Address': obj.contact_email,
                'Software Engineer - Estimated': (
                    obj.software_engineer_time_total and
                    str(obj.software_engineer_time_total) or ""),
                'Business Analyst Total Time': (
                    obj.business_analyst_time_total and
                    str(obj.business_analyst_time_total) or ""),
                'Transend EDI Total Support Time': (
                    obj.transend_edi_support_time_total and
                    str(obj.transend_edi_support_time_total) or ""),
                'System Engineer - Actual': (
                    obj.system_engineer_time_total and
                    str(obj.system_engineer_time_total) or ""),
                'Operations ROI - Estimated': (
                    obj.operations_roi_time and
                    str(obj.operations_roi_time) or ""),
            })
