import logging
from urllib.parse import quote_plus
from urllib.parse import urlparse

import requests

from Products.CMFCore.utils import getToolByName
from plone import api


logger = logging.getLogger('retrans.poi.gitlab')


def workflow_transition_for_files(object, event):
    """If a file/image is added to Issue, transition it to best
       intranet_workflow state."""
    parent = event.newParent
    if parent.portal_type in ('Issue'):
        tracker = parent.getTracker()
        state = api.content.get_state(tracker)
        portal = api.portal.get()
        wtool = getToolByName(portal, "portal_workflow")
        try:
            workflow = wtool.getChainForPortalType(object.portal_type)[0]
        except IndexError:
            return
        with api.env.adopt_user(username="sfupadmin"):
            if workflow == 'intranet_workflow':
                if state in ('closed', 'protected', 'restricted'):
                    transition = 'publish_internally'
                else:
                    api.content.transition(object, 'submit')
                    transition = 'publish_externally'
                api.content.transition(object, transition)


def handle_new_collection(object, event):
    """If a Collection is added to a Tracker,
       move it to the top of the folder order."""

    parent = event.newParent
    if parent.portal_type in ('Tracker'):
        parent.moveObjectToPosition(object.id, 0)


def update_gitlab_issue_state(issue, event):
    """ update GitLab Issue based on Poi Issue state change """

    # if the issue is not linked to a gitlab issue, we're done here
    if not getattr(issue, 'gitlab_issue_id', None):
        return

    issue_state = event.new_state.id
    # if it's not in progress, rejected, resolved or closed we're done here
    if issue_state not in ['in-progress', 'rejected', 'resolved', 'closed']:
        return

    # we need the current gitlab issue metadata
    user = api.user.get_current()
    api_token = user.getProperty('gitlab_api_token', None)
    tracker = issue.getTracker()
    if not tracker.repo_url or not api_token:
        return 'Please configure a Tracker URL and your GitLab API token.'
    url = urlparse(tracker.repo_url)
    api_root = '{}://{}/api/v4'.format(url.scheme, url.netloc)
    project_id = quote_plus(url.path.strip('/'))
    issue_url = '{}/projects/{}/issues/{}'.format(api_root,
        project_id, issue.gitlab_issue_id)
    try:
        api_response = requests.get(
            issue_url,
            headers={'PRIVATE-TOKEN': api_token,},
        )
        if api_response.ok:
            gitlab_issue = api_response.json()
    except Exception as e:
        logger.error("Can't find corresponding GitLab issue: {}".format(
            issue.gitlab_issue_id))
        return

    # get the issue labels
    current_labels = gitlab_issue.get('labels')

    # if poi issue is in progress and there are no labels on issue
    # add In Progress label
    if issue_state == 'in-progress' and len(current_labels) == 0:
        current_labels.append('In Progress')
        try:
            api_response = requests.put(
                issue_url,
                headers={'PRIVATE-TOKEN': api_token,},
                params={
                    'labels': ','.join(current_labels),
                }
            )
            if api_response.ok:
                return
            else:
                logger.error("Couldn't change GitLab Issue Labels.")
        except Exception as e:
            logger.error("Couldn't change GitLab Issue Labels.")
            return

    # if poi issue is rejected, resolved or closed and state labels
    # are not MTP or Merged Not Yet Closed then remove existing state
    # labels and Close the issue
    close_state_labels = [
        'Merged not yet closed',
        'MTP',
    ]
    close_poi_states = ['rejected', 'resolved', 'closed']
    close_overlap = list(set(close_state_labels) & set(current_labels))
    if issue_state in close_poi_states and len(close_overlap) == 0:
        try:
            api_response = requests.put(
                issue_url,
                headers={'PRIVATE-TOKEN': api_token,},
                params={
                    'labels': '',
                    'state_event': 'close',
                }
            )
            if api_response.ok:
                return
            else:
                logger.error("Couldn't change GitLab Issue Labels.")
        except Exception as e:
            logger.error("Couldn't change GitLab Issue Labels.")
            return
 